/***********************************************************************
 * Add the suitable linker script (e.g., p30f4011.gld) to the project. *
 *                                                                     *
 * If interrupts are not used, all code presented for that interrupt   *
 * can be removed or commented out with C-style comment declarations.  *
 *                                                                     *
 * For additional information about dsPIC architecture and language    *
 * tools, refer to the following documents:                            *
 *                                                                     *
 * MPLAB C30 Compiler User's Guide                        : DS51284    *
 * dsPIC 30F MPLAB ASM30, MPLAB LINK30 and Utilites                    *
 *                                           User's Guide : DS51317    *
 * Getting Started with dsPIC DSC Language Tools          : DS51316    *
 * dsPIC 30F Language Tools Quick Reference Card          : DS51322    *
 * dsPIC 30F 16-bit MCU Family Reference Manual           : DS70046    *
 * dsPIC 30F Motor Control and Power Conversion Family                 * 
 *                                             Data Sheet : DS70082    *
 * dsPIC 30F Programmer's Reference Manual                : DS70030    *
 *                                                                     *
 * Template file has been compiled with MPLAB C30 v1.30.               *
 *                                                                     *
 ***********************************************************************
 *                                                                     * 
 *    Author:            DDS                                           *
 *    Company:           University of Pisa                            * 
 *    File Description:  Motor DC Identification                       *
 *    Date:              -/-/2007                                      *
 *    File Version:      1.0                                           *
 *    Other Files Required: p33FJ256MC710.gld                          *
 *                			                                           *
 *                                                                     *
 ***********************************************************************
 *                                                                     *
 * Other Comments:                                                     *
 *                                                                     *
 * 1) C attributes, designated by the __attribute__ keyword, provide a *
 *    means to specify various characteristics of a variable or        *
 *    function, such as where a particular variable should be placed   *
 *    in memory, whether the variable should be aligned to a certain   *
 *    address boundary, whether a function is an Interrupt Service     *
 *    Routine (ISR), etc. If no special characteristics need to be     *
 *    specified for a variable or function, then attributes are not    *
 *    required. For more information about attributes, refer to the    *
 *    C30 User's Guide.                                                *
 *                                                                     *
 * 2) The __space__(xmemory) and __space__(ymemory) attributes are     *
 *    used to place a variable in X data space and Y data space,       *
 *    respectively. Variables accessed by dual-source DSP instructions *
 *    must be defined using these attributes.                          *
 *                                                                     *
 * 3) The aligned(k) attribute, used in variable definitions, is used  *
 *    to align a variable to the nearest higher 'k'-byte address       * 
 *    boundary. 'k' must be substituted with a suitable constant       *
 *    number when the ModBuf_X(k) or ModBuf_Y(k) macro is invoked.     *
 *    In most cases, variables are aligned either to avoid potential   *
 *    misaligned memory accesses, or to configure a modulo buffer.     *                           
 *                                                                     *
 * 4) The __interrupt__ attribute is used to qualify a function as an  *
 *    interrupt service routine. An interrupt routine can be further   *
 *    configured to save certain variables on the stack, using the     *
 *    __save__(var-list) directive.                                    *
 *                                                                     *
 * 5) The __shadow__ attribute is used to set up any function to       *  
 *    perform a fast context save using shadow registers.              *
 *                                                                     *
 * 6) Note the use of double-underscores (__) at the start and end of  *
 *    all the keywords mentioned above.                                *
 *                                                                     *
 **********************************************************************/      

#include "p33FJ256MC710.h"
#include <math.h>
#include "pwm.h"
#include "uart.h"
#include "adc.h"
#include "timer.h"   
                                                                              
_FOSCSEL(  FNOSC_PRIPLL );                                                                          
_FOSC( FCKSM_CSDCMD  & OSCIOFNC_OFF & POSCMD_XT );
_FWDT( FWDTEN_OFF );
_FGS( GSS_OFF & GCP_OFF & GWRP_OFF);               /* Disable Code Protection */

/* ************ Define constants here ************* */

#define dim_data 26000 	// -> n_prbs*periods*camp

/************* START OF GLOBAL DEFINITIONS **********/
/* Tcy = 8e-8 sec; */

int DCpwm = 0;        	    	/* PWM duty cycle */ 
unsigned int Tsample = 799;				/* Tempo di campionamento -> Timer23 (4.096e-5 sec) Prescaler -> 1:1*/
unsigned int c_lim = 255;            	/* Current limit (with scale-factor Rs=2) and digital scale factor 2*/
unsigned int c_danger = 350;			/* 350 -> 2.25A-2.3A */
unsigned char Buf[4];              		/* UART1 Rx Buffer */ 
char st[4];								/* UART1 Tx Buffer */

unsigned int icount = 0, switch_adc = 0; 
unsigned int camp = 20, tad = 5;
unsigned long int Tpwm = 0;				/* pre = 64; ATT!!! ...like timer23 prescaler... */
unsigned char *iav, *vav;

float uin1 = 0.0;
int errc1 = 0, errc0=0; 
/*
int ref = 150; 	// 2A
int Vmin = 40, Vmax = 183; 			//1V, 5.7V
int usat = 0, gap = 143;  			//Vmax-Vmin = 183.0-40.0; 1000 7 0.002
									//Vmax-Vmin = 195.0-35.0; 800 5 0.002
/************** END OF GLOBAL DEFINITIONS ***********/


void ADC1_config(void);

void PWM_config(unsigned int dc, unsigned int Ts);

void UART1_config(void);

void TIMER23_config(unsigned long int match_value);

void send_by_UART1(unsigned int *buffer, unsigned int n);

void __attribute__((__interrupt__)) _ADC1Interrupt(void);

void __attribute__((__interrupt__)) _U1RXInterrupt(void);

void __attribute__((__interrupt__)) _T5Interrupt(void);

void reset_counters();

void stop_simulation();


/************* START OF MAIN FUNCTION ***************/

int main ( void )
{ 
  	unsigned char ias[dim_data]; 
	//unsigned char vas[dim_data];

	UART1_config();         // Initialize the UART module to receive data.
  	TRISEbits.TRISE2 = 0;   // Enable signal on RE2. 
	iav = ias;
	//vav = vas;
	PORTEbits.RE2 = 0;      // H-Bridge disabled.	

    st[0] = 0xAA;			// Start Signal.
  	st[1] = 0xBB;
  	st[2] = 0xCC;
  	st[3] = '\0'; 
  	send_by_UART1((unsigned int *)st,3); // Characters transimitted in hex.
  
  	while(1)
    	if(icount > 25600)
      		stop_simulation();                 
  	return 0;
}


/****** START OF INTERRUPT SERVICE ROUTINES *********/
/* Replace the interrupt function names with the    */
/* appropriate names depending on interrupt source. */
/* The names of various interrupt functions for     */
/* each device are defined in the linker script.    */
/* Interrupt Service Routine 1                      */
/* No fast context save, and no variables stacked   */
/****************************************************/

/* This is: ADC1 ISR */
void __attribute__((__interrupt__)) _ADC1Interrupt(void) 
{
	/*  Clear the A/D Interrupt flag bit or else the CPU will
        keep vectoring back to the ISR*/
    IFS0bits.AD1IF = 0;
	
	//if(switch_adc == 0){
		// Corrente di armatura (output):
		if(ADC1BUF0 > c_danger) stop_simulation();
		iav[icount] = ADC1BUF0/2; 		 			// ATT!!! scale-factor = 2...
		errc0 = 200-ADC1BUF0;
		//errc0 = 0;
		uin1 += (float)errc1*0.005;					// Regulator. Double: [V], [A] u1 +=2e-3*err1
		
		errc1 = errc0;
		DCpwm = ( ((long int)uin1)-35 )*5;
		DCpwm = (DCpwm<50)?1:( (DCpwm>800)?800:DCpwm );
		*(&PDC1+2) = DCpwm*2-1;		// Set DC -> Vmax = 5.7V , Vmin=1V					
	//}
	/*
	if(switch_adc == 1) // Tensione di armatura (input):
		// if( ADC1BUF0 > c_lim ) stop_simulation();
		vav[icount++] = ADC1BUF0; 					// ATT!!! scale-factor = 2...
	*/
	icount++;
	//switch_adc ^= 1;
}

/* This is: UART1 receive ISR */
void __attribute__((__interrupt__)) _U1RXInterrupt(void)
{
    unsigned long int x,y,z;
	IFS0bits.U1RXIF = 0;
    int i=0;
    for(i=0; i<4 && DataRdyUART1();i++)
        Buf[i] = ReadUART1();  
    /* Prompt: AA xx xx xx -> Turn on the real simulation and set the sample-time;
               BB 00 00 00 -> Turn off the real simulation;
               CC .. .. .. -> Start Control...
               DD 00 xx xx -> New Duty Cycle for the PWM Module;
               EE .. .. .. -> Nothing;
               FF xx xx xx -> Nothing; */
 
     	switch (Buf[0])
     	{
			/*******************************/
    		case 0xAA: 
			  if(PORTEbits.RE2 != 1){
				reset_counters();
            	ADC1_config();					// Configure the A/D converter.
			    x = Buf[1];
				y = Buf[2];
				z = Buf[3];
				x = (x<<16) + (y<<8) + z;
                if(x!=0)
                	TIMER23_config(x);       	// Timer23(32 bit) configuration.
				else 
					TIMER23_config(Tsample);
				Tpwm = Tsample;					// Calculate the PWM period.
				PWM_config((unsigned int)DCpwm,Tpwm);			// Configure the PWM module.
                st[0] = (x>>16) & 0x00FF;
                st[1] = (x>>8) & 0x00FF;
				st[2] = x & 0x00FF;
                send_by_UART1((unsigned int *)st,3);
                while(BusyUART1());
				PORTEbits.RE2 = 1;    	// H-Bridge enabled.	
			  }	break;
            /*******************************/                 
			case 0xBB: 
				stop_simulation();
            	break;
			/*******************************/
        	case 0xCC: // Start Control...
				T2CONbits.TON= 1;		// Turn on Timer23. 
				AD1CON1bits.ADON = 1; 	// Turn on ADC1
				PTCONbits.PTEN= 1;    	// Turn on PWM. 
				break;  
      		/*******************************/
			case 0xDD:
				x = (Buf[2]<<8) + Buf[3];
            	if(x > 0)  
            		DCpwm = (unsigned int)x;
            	if(AD1CON1bits.ADON == 0){
            		st[0] = (char) ((DCpwm>>8) & 0x00FF);
                	st[1] = (char) (DCpwm & 0x00FF);
                	st[2] = '\0';
                	send_by_UART1((unsigned int *)st,2);
                	while(BusyUART1());
            	}
				break;
			/*******************************/	
			case 0xEE:
				break;
			/*******************************/
			case 0xFF: // Duty Cycle settings.
            	break;
			/*******************************/
        	default:     
				if(AD1CON1bits.ADON == 0){
            		char s[5];
                	s[0] = (char) 0xFF;
                	s[1] = (char) 0xFF;
                	s[2] = (char) 0xFF;
                	s[3] = (char) 0xFF;
                	s[4] = '\0';
                	send_by_UART1((unsigned int *)s,4);
                	while(BusyUART1());
            	}
			/*******************************/
	 	}
}

/****************************** User Functions ********************************/

/* Stop the simulation. */
void stop_simulation()
{ 
	int i,k;
	CloseADC1();          	// Turn off ADC1.
    PORTEbits.RE2 = 0;    	// H-Bridge disabled. 
    CloseMCPWM();         	// Turn on PWM.
    CloseTimer23();       	// Turn off Timer23. 
    while(BusyUART1());

   	// Final results.
    st[0] = (char) 0xEE;
    st[1] = (char) 0xEE;
    st[2] = (char) 0xEE;
    st[3] = (char) 0xEE;
    send_by_UART1((unsigned int *)st,4);
    while(BusyUART1());
    st[0] = (char) 0xDD;
    st[1] = (char) 0xDD;
    st[2] = (char) ((icount>>8) & 0x00FF);
    st[3] = (char) (icount & 0x00FF);
    send_by_UART1((unsigned int *)st,4);
    while(BusyUART1());
	
	// Border between the vectors (EE EE EE EE EE EE EE EE):
    st[0] = (char) 0xEE;
    st[1] = (char) 0xEE;
    st[2] = (char) 0xEE;
    st[3] = (char) 0xEE;
    send_by_UART1((unsigned int *)st,4);
	while(BusyUART1());
    send_by_UART1((unsigned int *)st,4);

	// Ia:
	for(i=0; i<icount-1; i++){
	    while(BusyUART1());
   		// Final Ia vector:
		for(k=0;k<1000;k++);
    	st[0] = (char) 0xFF;
    	st[1] = (char) (iav[i] & 0x00FF);
    	send_by_UART1((unsigned int *)st,2);
	}
	while(BusyUART1());
	/*
	// Border between the vectors (EE EE EE EE EE EE EE EE):
    st[0] = (char) 0xEE;
    st[1] = (char) 0xEE;
    st[2] = (char) 0xEE;
    st[3] = (char) 0xEE;
    send_by_UART1((unsigned int *)st,4);
	while(BusyUART1());
    send_by_UART1((unsigned int *)st,4);

	// Va:
	for(i=0; i<icount-1; i++){
	    while(BusyUART1());
   		// Final Va vector:
		for(k=0;k<1000;k++);
    	st[0] = (char) 0xFF;
    	st[1] = (char) (vav[i] & 0x00FF);
    	send_by_UART1((unsigned int *)st,2);
	}
	while(BusyUART1());
	*/

	// The end:
	st[0] = (char) 0xEE;
    st[1] = (char) 0xEE;
    st[2] = (char) 0xEE;
    st[3] = (char) 0xEE;
    send_by_UART1((unsigned int *)st,4);
	while(BusyUART1());
    send_by_UART1((unsigned int *)st,4);
	while(BusyUART1());

	reset_counters();
    return;     
}

void reset_counters(){
	icount = 0;
	switch_adc = 0;
	return;
}






